/**
 * provides a template wrapper around an application for including CLUE. Includes the common frame for switching modes, provenance, and story visualizations
 *
 * Created by Samuel Gratzl on 27.08.2015.
 */
import * as tslib_1 from "tslib";
import * as template from 'html-loader!./_template.html';
import { hash, mixin } from 'phovea_core/src/index';
import { create as createHeader, AppHeaderLink } from 'phovea_ui/src/header';
import { create as createData } from 'phovea_core/src/data';
import { list as listVis } from 'phovea_core/src/vis';
import { MixedStorageProvenanceGraphManager } from 'phovea_core/src/provenance';
import { event as d3event, select, selectAll, time, mouse as d3mouse, behavior } from 'd3';
import * as $ from 'jquery';
import { create as createSelection } from './selection';
import * as cmode from './mode';
import { create as createProvVis } from './provvis';
import { create as createStoryVis } from './storyvis';
import { EventHandler } from 'phovea_core/src/event';
import { create as createAnnotation } from './annotation';
import { bindLoginForm, form as loginForm, logout } from 'phovea_security_flask/src/login';
import { retrieve } from 'phovea_core/src/session';
import { generateDialog, areyousure } from 'phovea_ui/src/dialogs';
import { setLoggedIn, isLoggedIn } from './user';
var CLUEGraphManager = (function () {
    function CLUEGraphManager(manager) {
        this.manager = manager;
        //selected by url
    }
    CLUEGraphManager.setGraphInUrl = function (value) {
        hash.removeProp('clue_slide', false);
        hash.removeProp('clue_state', false);
        hash.setProp('clue_graph', value);
    };
    CLUEGraphManager.prototype.newRemoteGraph = function () {
        if (isLoggedIn()) {
            CLUEGraphManager.setGraphInUrl('new_remote');
            window.location.reload();
        }
    };
    CLUEGraphManager.prototype.newGraph = function () {
        CLUEGraphManager.setGraphInUrl('new');
        window.location.reload();
    };
    CLUEGraphManager.prototype.loadGraph = function (desc) {
        // reset
        CLUEGraphManager.setGraphInUrl(desc.id);
        window.location.reload();
    };
    Object.defineProperty(CLUEGraphManager.prototype, "storedSlide", {
        get: function () {
            return hash.getInt('clue_slide', null);
        },
        set: function (value) {
            if (value !== null) {
                hash.setInt('clue_slide', value);
            }
            else {
                hash.removeProp('clue_slide');
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CLUEGraphManager.prototype, "storedState", {
        get: function () {
            return hash.getInt('clue_state', null);
        },
        set: function (value) {
            if (value !== null) {
                hash.setInt('clue_state', value);
            }
            else {
                hash.removeProp('clue_state');
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CLUEGraphManager.prototype, "isAutoPlay", {
        get: function () {
            return hash.is('clue_autoplay');
        },
        enumerable: true,
        configurable: true
    });
    CLUEGraphManager.prototype.list = function () {
        return this.manager.list();
    };
    CLUEGraphManager.prototype.delete = function (graph) {
        return this.manager.delete(graph);
    };
    CLUEGraphManager.prototype.importGraph = function (dump, remote) {
        var _this = this;
        if (remote === void 0) { remote = false; }
        (remote ? this.manager.importRemote(dump) : this.manager.importLocal(dump)).then(function (graph) {
            _this.loadGraph(graph.desc);
        });
    };
    CLUEGraphManager.prototype.setGraph = function (graph) {
        hash.setProp('clue_graph', graph.desc.id);
    };
    CLUEGraphManager.prototype.choose = function (list) {
        var loggedIn = retrieve('logged_in', false) === true;
        var graph = hash.getProp('clue_graph', null);
        if (graph === 'new_remote' && loggedIn) {
            return this.manager.createRemote();
        }
        if (graph === null || graph === 'new') {
            return this.manager.createLocal();
        }
        var desc = list.find(function (d) { return d.id === graph; });
        if (desc) {
            if (desc.local || loggedIn) {
                return this.manager.get(desc);
            }
            return this.manager.cloneLocal(desc);
        }
        return this.manager.create();
    };
    CLUEGraphManager.prototype.loadOrClone = function (graph, isSelect) {
        var _this = this;
        if (isSelect) {
            this.loadGraph(graph);
        }
        else {
            this.manager.cloneLocal(graph).then(function (graph) { return _this.loadGraph(graph.desc); });
        }
    };
    return CLUEGraphManager;
}());
export { CLUEGraphManager };
/**
 * create the provenance graph selection dropdown and handles the graph selection
 * @param manager
 * @param $ul
 * @returns {Promise<U>}
 */
function chooseProvenanceGraph(manager, $ul) {
    //new button
    $ul.select('#provenancegraph_new_remote').on('click', function () {
        d3event.preventDefault();
        manager.newRemoteGraph();
    });
    //new local
    $ul.select('#provenancegraph_new').on('click', function () {
        d3event.preventDefault();
        manager.newGraph();
    });
    selectAll('#provenancegraph_import, #provenancegraph_import_remote').on('click', function () {
        var e = d3event;
        e.preventDefault();
        e.stopPropagation();
        var remote = this.id === 'provenancegraph_import_remote';
        //import dialog
        var d = generateDialog('Select File', 'Upload');
        d.body.innerHTML = "<input type=\"file\" placeholder=\"Select File to Upoad\">";
        select(d.body).select('input').on('change', function () {
            var file = d3event.target.files[0];
            var reader = new FileReader();
            reader.onload = function (e) {
                var dataS = e.target.result;
                var dump = JSON.parse(dataS);
                manager.importGraph(dump, remote);
            };
            // Read in the image file as a data URL.
            reader.readAsText(file);
        });
        d.show();
    });
    return manager.list().then(function (list) {
        var $list = $ul.select('#provenancegraph_list').selectAll('li.graph').data(list);
        $list.enter().insert('li', ':first-child').classed('graph', true).html(function (d) { return "<a href=\"#clue_graph=" + d.id + "\"><i class=\"fa fa-code-fork fa-rotate-180\" aria-hidden=\"true\"></i> " + d.name + " </a>"; }).select('a').on('click', function (d) {
            d3event.preventDefault();
            manager.loadGraph(d);
        });
        var format = time.format.utc('%Y-%m-%dT%H:%M');
        $('#provenancegraph_list li.graph a').popover({
            html: true,
            placement: 'left',
            trigger: 'manual',
            title: function () {
                var graph = select(this).datum();
                return "" + graph.name;
            },
            content: function () {
                var graph = select(this).datum();
                var creator = graph.creator;
                var description = graph.description || '';
                var ts = graph.ts ? format(new Date(graph.ts)) : 'Unknown';
                var nnodes = graph.size[0];
                var nedges = graph.size[1];
                //const locked = false;
                var $elem = $("\n            <div class=\"container-fluid\">\n            <div class=\"row\">\n                <div class=\"col-sm-3\">creator:</div>\n                <div class=\"col-sm-9\">" + creator + "</div>\n            </div>\n            <div class=\"row\">\n                <div class=\"col-sm-3\">created:</div>\n                <div class=\"col-sm-9\">" + ts + "</div>\n            </div>\n            <div class=\"row\">\n                <div class=\"col-sm-3\">description:</div>\n                <div class=\"col-sm-9\">" + description + "</div>\n            </div>\n            <div class=\"row\">\n                <div class=\"col-sm-3\">nodes/edges:</div>\n                <div class=\"col-sm-9\">" + nnodes + "/" + nedges + "</div>\n            </div>\n            <div class=\"row\">\n                <div class=\"col-sm-12 text-right\">\n                    <button class=\"btn btn-primary\" " + (retrieve('logged_in', false) !== true && !graph.local ? 'disabled="disabled"' : '') + " data-action=\"select\" data-toggle=\"modal\"><span class=\"fa fa-folder-open\" aria-hidden=\"true\"></span> Select</button>\n                    <button class=\"btn btn-primary\" data-action=\"clone\" data-toggle=\"modal\"><span class=\"fa fa-clone\" aria-hidden=\"true\"></span> Clone</button>\n                    <button class=\"btn btn-danger\" " + (retrieve('logged_in', false) !== true && !graph.local ? 'disabled="disabled"' : '') + " data-toggle=\"modal\"><i class=\"fa fa-trash\" aria-hidden=\"true\"></i> Delete</button>\n                </div>\n            </div>\n        </div>");
                $elem.find('button.btn-danger').on('click', function () {
                    areyousure('Are you sure to delete: "' + graph.name + '"').then(function (deleteIt) {
                        if (deleteIt) {
                            manager.delete(graph);
                        }
                    });
                });
                $elem.find('button.btn-primary').on('click', function () {
                    var isSelect = this.dataset.action === 'select';
                    manager.loadOrClone(graph, isSelect);
                    return false;
                });
                return $elem;
            }
        }).parent().on({
            mouseenter: function () {
                $(this).find('a').popover('show');
            },
            mouseleave: function () {
                $(this).find('a').popover('hide');
            }
        });
        return manager.choose(list);
    }).then(function (graph) {
        manager.setGraph(graph);
        $ul.select('#provenancegraph_name').text(graph.desc.name);
        return graph;
    });
}
/**
 * injection for headless support
 * @param wrapper
 */
function injectHeadlessSupport(wrapper) {
    var w = window;
    w.__caleydo = w.__caleydo || {};
    w.__caleydo.clue = wrapper;
    wrapper.on('jumped_to', function () {
        setTimeout(function () {
            document.body.classList.add('clue_jumped');
            prompt('clue_done_magic_key', 'test');
        }, 5000);
    });
}
function injectParentWindowSupport(wrapper) {
    var w = window;
    w.__caleydo = w.__caleydo || {};
    w.__caleydo.clue = wrapper;
    //initial jump
    var jumpListener = function (s) {
        window.top.postMessage({ type: 'caleydo', clue: 'jumped_to_initial' }, '*');
        wrapper.off('jumped_to', jumpListener);
    };
    wrapper.on('jumped_to', jumpListener);
    window.addEventListener('message', function (event) {
        var s = event.source, d = event.data;
        if (d.type !== 'caleydo' || !d.clue) {
            return;
        }
        if (d.clue === 'jump_to') {
            wrapper.jumpToState(d.state).then(function () {
                s.postMessage({ type: 'caleydo', clue: 'jumped_to', state: d.state, ref: d.ref }, '*');
            }).catch(function () {
                s.postMessage({ type: 'caleydo', clue: 'jump_to_error', state: d.state, ref: d.ref }, '*');
            });
        }
        else if (d.clue === 'show_slide') {
            wrapper.jumpToStory(d.slide).then(function () {
                s.postMessage({ type: 'caleydo', clue: 'show_slide', slide: d.slide, ref: d.ref }, '*');
            }).catch(function () {
                s.postMessage({ type: 'caleydo', clue: 'show_slide_error', slide: d.slide, ref: d.ref }, '*');
            });
        }
        else if (d.clue === 'next_slide') {
            wrapper.nextSlide().then(function () {
                s.postMessage({ type: 'caleydo', clue: 'next_slide', ref: d.ref }, '*');
            });
        }
        else if (d.clue === 'previous_slide') {
            wrapper.previousSlide().then(function () {
                s.postMessage({ type: 'caleydo', clue: 'previous_slide', ref: d.ref }, '*');
            });
        }
    });
}
var CLUEWrapper = (function (_super) {
    tslib_1.__extends(CLUEWrapper, _super);
    function CLUEWrapper(body, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.options = {
            app: 'CLUE',
            application: '/clue',
            id: 'clue',
            recordSelectionTypes: 'selected',
            animatedSelections: false,
            thumbnails: true,
            appLink: new AppHeaderLink('CLUE'),
            provVisCollapsed: false,
            headerOptions: {},
            loginForm: String(loginForm)
        };
        var that = _this;
        mixin(_this.options, options);
        // replace content with the template
        body.innerHTML = String(template);
        //special flag for rendering server side screenshots
        if (hash.is('clue_headless')) {
            console.log('init headless mode');
            injectHeadlessSupport(_this);
            select('body').classed('headless', true);
        }
        if (hash.is('clue_contained')) {
            console.log('init contained mode');
            injectParentWindowSupport(_this);
            select('body').classed('headless', true);
        }
        //load all available provenance graphs
        _this.manager = new MixedStorageProvenanceGraphManager({
            prefix: _this.options.id,
            storage: sessionStorage,
            application: _this.options.application
        });
        _this.clueManager = new CLUEGraphManager(_this.manager);
        //create the common header
        var headerOptions = mixin(_this.options.headerOptions, {
            showOptionsLink: true,
            appLink: _this.options.appLink
        });
        _this.header = createHeader(body.querySelector('div.box'), headerOptions);
        _this.header.wait();
        _this.createLogin();
        {
            //add provenance graph management menu entry
            var ul = document.createElement('ul');
            var $ul = select(ul)
                .attr('class', 'nav navbar-nav navbar-right')
                .attr('data-clue', 'provenanceGraphList')
                .html("<li class=\"dropdown\">\n          <a class=\"active\" href=\"#\" class=\"dropdown-toggle\" data-toggle=\"dropdown\" role=\"button\" aria-haspopup=\"true\" aria-expanded=\"false\">\n            <i class=\"fa fa-code-fork fa-lg fa-rotate-180 fa-fw\"></i>\n          </a>\n          <ul class=\"dropdown-menu\" id=\"provenancegraph_list\">\n            <li role=\"separator\" class=\"divider\"></li>\n            <li>\n              <a href=\"#\" id=\"provenancegraph_import\">\n                <i class=\"fa fa-upload\" aria-hidden=\"true\"></i> Import ...\n              </a>\n            </li>\n            <li>\n              <a href=\"#\" class=\"login_required disabled\" disabled=\"disabled\" id=\"provenancegraph_import_remote\">\n                <i class=\"fa fa-cloud-upload\" aria-hidden=\"true\"></i> Import Remote ...\n              </a>\n            </li>\n            <li>\n              <a href=\"#\" id=\"provenancegraph_export\">\n                <i class=\"fa fa-download\" aria-hidden=\"true\"></i> Export ...\n              </a>\n            </li>\n            <li role=\"separator\" class=\"divider\"></li>\n            <li>\n              <a href=\"#\" id=\"provenancegraph_new\">\n                <i class=\"fa fa-plus-circle\" aria-hidden=\"true\"></i> New ...\n              </a>\n            </li>\n            <li>\n              <a href=\"#\" class=\"login_required disabled\" disabled=\"disabled\" id=\"provenancegraph_new_remote\">\n                <i class=\"fa fa-cloud\" aria-hidden=\"true\"></i> New Remote...\n              </a>\n            </li>\n          </ul>\n        </li>");
            _this.header.insertCustomRightMenu(ul);
            select('#provenancegraph_export').on('click', function () {
                var e = d3event;
                e.preventDefault();
                e.stopPropagation();
                _this.graph.then(function (g) {
                    console.log(g);
                    var r = g.persist();
                    console.log(r);
                    var str = JSON.stringify(r, null, '\t');
                    //create blob and save it
                    var blob = new Blob([str], { type: 'application/json;charset=utf-8' });
                    var a = new FileReader();
                    a.onload = function (e) { return window.open(e.target.result, '_blank'); };
                    a.readAsDataURL(blob);
                });
                return false;
            });
            select(_this.header.optionsDialog)
                .append('button').text('Show Provenance Graph')
                .attr('class', 'btn btn-default')
                .on('click', function () {
                _this.graph.then(function (g) {
                    return createData({
                        id: g.desc.id,
                        name: g.desc.name,
                        description: '',
                        fqname: g.desc.fqname,
                        type: 'graph',
                        storage: 'given',
                        graph: g.backend,
                        creator: 'Anonymous',
                        ts: Date.now()
                    });
                })
                    .then(function (proxy) {
                    var l = listVis(proxy);
                    if (l.length <= 0) {
                        return;
                    }
                    l[0].load().then(function (force) {
                        var p = generateDialog('Provenance Graph');
                        force.factory(proxy, p.body);
                        p.show();
                    });
                });
                return false;
            });
            _this.graph = chooseProvenanceGraph(_this.clueManager, $ul);
        }
        //cmode.create(body.querySelector('#modeselector'));
        var modeselector = body.querySelector('header');
        modeselector.className += 'clue-modeselector';
        cmode.createButton(modeselector, {
            size: 'sm'
        });
        //cmode.createSlider(body.querySelector('#modeselector'));
        _this.$main = select(body).select('main');
        _this.graph.then(function (graph) {
            graph.on('sync_start,sync', function (event) {
                select('nav span.glyphicon-cog').classed('fa-spin', event.type !== 'sync');
            });
            if (_this.options.recordSelectionTypes) {
                //record selections of the given type
                createSelection(graph, _this.options.recordSelectionTypes, {
                    filter: function (idtype) {
                        return idtype && idtype.name[0] !== '_';
                    },
                    animated: _this.options.animatedSelections
                });
            }
            _this.$mainRef = graph.findOrAddObject(_this.$main, 'Application', 'visual');
            var r = createAnnotation(_this.$main.node(), graph);
            /*const seditor = storyeditor.create(graph, body.querySelector('#storyeditor'), {
             editor: r.edit
             });
             */
            createProvVis(graph, body.querySelector('div.content'), {
                thumbnails: _this.options.thumbnails,
                provVisCollapsed: _this.options.provVisCollapsed
            });
            _this.storyvis = createStoryVis(graph, body.querySelector('div.content'), {
                render: r.render,
                thumbnails: _this.options.thumbnails
            });
            graph.on('select_slide_selected', function (event, state) {
                select('aside.annotations').style('display', state ? null : 'none');
            });
            select('aside.annotations > div:first-of-type').call(behavior.drag().on('drag', function () {
                var mouse = d3mouse(this.parentElement.parentElement);
                select(this.parentElement).style({
                    left: mouse[0] + 'px',
                    top: mouse[1] + 'px'
                });
            }));
            selectAll('aside.annotations button[data-ann]').on('click', function () {
                var create = this.dataset.ann;
                var ann;
                switch (create) {
                    case 'text':
                        ann = {
                            type: 'text',
                            pos: [10, 10],
                            text: ''
                        };
                        break;
                    case 'arrow':
                        ann = {
                            type: 'arrow',
                            pos: [10, 10],
                            at: [200, 200]
                        };
                        //that.data.appendToStory(that.story.story, that.data.makeTextStory('Unnamed');
                        //this.actStory.addText();
                        break;
                    case 'frame':
                        ann = {
                            type: 'frame',
                            pos: [10, 10],
                            size: [20, 20]
                        };
                        break;
                }
                if (that.storyvis && ann) {
                    that.storyvis.pushAnnotation(ann);
                }
            });
            graph.on('switch_state', function (event, state) {
                _this.clueManager.storedState = state ? state.id : null;
            });
            graph.on('select_slide_selected', function (event, state) {
                _this.clueManager.storedSlide = state ? state.id : null;
            });
            {
                var $right_1 = $('aside.provenance-layout-vis');
                var $rightStory_1 = $(_this.storyvis.node);
                _this.propagate(cmode, 'modeChanged');
                var update_1 = function (newMode) {
                    $('body').attr('data-clue', newMode.toString());
                    //$('nav').css('background-color', d3.rgb(255 * new_.exploration, 255 * new_.authoring, 255 * new_.presentation).darker().darker().toString());
                    if (newMode.presentation > 0.8) {
                        $right_1.animate({ width: 'hide' }, 'fast');
                    }
                    else {
                        $right_1.animate({ width: 'show' }, 'fast');
                    }
                    if (newMode.exploration > 0.8) {
                        $rightStory_1.animate({ width: 'hide' }, 'fast');
                    }
                    else {
                        $rightStory_1.animate({ width: 'show' }, 'fast');
                    }
                };
                cmode.on('modeChanged', function (event, newMode) { return update_1(newMode); });
                _this.fire('modeChanged', cmode.getMode());
                update_1(cmode.getMode());
            }
            //d3.select('#attachScreenshot').on('click', () => {
            //  const main = <HTMLElement>(document.querySelector('main *[data-main]') || document.querySelector('main'));
            //  const bounds = C.bounds(main);
            //  this.header.wait();
            //  screenshot.takeCanvas(main, [bounds.w/2, bounds.h/2]).then((canvas) => {
            //    this.header.ready();
            //    graph.act.setAttr('screenshot', screenshot.toString(canvas));
            //    graph.act.setAttr('thumbnail', screenshot.toString(screenshot.createThumbnailCanvas(canvas, 128)));
            //  }).catch((error) => {
            //    console.log(error);
            //  });
            //});
            select('#bookmarkState').on('click', function () {
                graph.act.setAttr('starred', true);
            });
            select('#attachNote form').on('submit', function () {
                var note = select('#attachNote_note').property('value');
                var e = d3event;
                graph.act.setAttr('note', note);
                $('#attachNote').modal('hide');
                document.querySelector('#attachNote form').reset();
                e.preventDefault();
                e.stopPropagation();
            });
            //undo the step
            select('#undoStep').on('click', function () {
                graph.undo();
            });
            //undo using ctrl-z
            select(document).on('keydown.player', function () {
                var k = d3event;
                if (k.keyCode === 90 && k.ctrlKey) {
                    //ctrl-z
                    k.preventDefault();
                    graph.undo();
                }
                else if (k.keyCode === 37 && k.ctrlKey) {
                    //left arrow 	37
                    cmode.setMode(cmode.modes.Exploration);
                }
                else if ((k.keyCode === 38 || k.keyCode === 40) && k.ctrlKey) {
                    //up arrow 	38
                    //down arrow 	40
                    cmode.setMode(cmode.modes.Authoring);
                }
                else if (k.keyCode === 39 && k.ctrlKey) {
                    //right arrow 	39
                    cmode.setMode(cmode.modes.Presentation);
                }
            });
            _this.fire('loaded_graph', graph);
            _this.header.ready();
        });
        return _this;
    }
    CLUEWrapper.prototype.createLogin = function () {
        {
            var ul = document.createElement('ul');
            ul.classList.add('nav', 'navbar-nav', 'navbar-right');
            ul.innerHTML = "\n      <li id=\"login_menu\">\n        <a data-toggle=\"modal\" data-target=\"#loginDialog\" href=\"#\">\n        <i class=\"fa fa-user fa-fw\" aria-hidden=\"true\"></i>\n        </a></li>\n        <li style=\"display: none\" class=\"dropdown\" id=\"user_menu\">\n            <a href=\"#\" class=\"dropdown-toggle\" data-toggle=\"dropdown\" role=\"button\" aria-haspopup=\"true\"\n               aria-expanded=\"false\"><i class=\"fa fa-user\" aria-hidden=\"true\"></i> Unknown</a>\n            <ul class=\"dropdown-menu\">\n                <li role=\"separator\" class=\"divider\"></li>\n                <li><a href=\"#\" id=\"logout_link\">Logout</a></li>\n            </ul>\n        </li>";
            this.header.insertCustomRightMenu(ul);
        }
        var that = this;
        {
            var $form_1 = $('#loginDialog div.modal-body').html(this.options.loginForm).find('form');
            var $alert_1 = $form_1.parent().find('div.alert');
            $alert_1.hide();
            bindLoginForm($form_1[0], function (error, user) {
                setLoggedIn((!error && user) ? true : false, user);
                if (!error && user) {
                    $('#login_menu').hide();
                    var $base = $('#user_menu').show();
                    $form_1.removeClass('has-error');
                    $base.find('> a:first').text(user.name);
                    $('#loginDialog').modal('hide');
                    $('.login_required.disabled').removeClass('disabled').attr('disabled', null);
                }
                else {
                    that.header.ready();
                    $form_1.addClass('has-error');
                    $alert_1.html(error).show();
                }
            });
        }
        $('#logout_link').on('click', function () {
            that.header.wait();
            logout().then(function () {
                setLoggedIn(false);
                $('#user_menu').hide();
                $('#login_menu').show();
                $('.login_required').addClass('disabled');
                that.header.ready();
            });
        });
    };
    CLUEWrapper.prototype.nextSlide = function () {
        var _this = this;
        return this.graph.then(function (graph) {
            return _this.storyvis.player.forward();
        });
    };
    CLUEWrapper.prototype.previousSlide = function () {
        var _this = this;
        return this.graph.then(function (graph) {
            return _this.storyvis.player.backward();
        });
    };
    CLUEWrapper.prototype.jumpToStory = function (story) {
        var _this = this;
        console.log('jump to stored story', story);
        return this.graph.then(function (graph) {
            var s = graph.getSlideById(story);
            if (s) {
                console.log('jump to stored story', s.id);
                _this.storyvis.switchTo(s);
                var next = void 0;
                if (_this.clueManager.isAutoPlay) {
                    _this.storyvis.player.start();
                    next = Promise.resolve();
                }
                else {
                    next = _this.storyvis.player.render(s);
                }
                return next.then(function () {
                    _this.fire('jumped_to', s);
                    _this.header.ready();
                    return _this;
                });
            }
            _this.fire('jumped_to', null);
            _this.header.ready();
            return Promise.reject('story not found');
        });
    };
    CLUEWrapper.prototype.jumpToState = function (state) {
        var _this = this;
        console.log('jump to stored state', state);
        return this.graph.then(function (graph) {
            var s = graph.getStateById(state);
            if (s) {
                console.log('jump to stored', s.id);
                return graph.jumpTo(s).then(function () {
                    console.log('jumped to stored', s.id);
                    _this.fire('jumped_to', s);
                    _this.header.ready();
                    return _this;
                });
            }
            _this.fire('jumped_to', null);
            _this.header.ready();
            return Promise.reject('state not found');
        });
    };
    CLUEWrapper.prototype.jumpToStored = function () {
        //jump to stored state
        var targetStory = this.clueManager.storedSlide;
        if (targetStory !== null) {
            return this.jumpToStory(targetStory);
        }
        var targetState = this.clueManager.storedState;
        if (targetState !== null) {
            return this.jumpToState(targetState);
        }
        this.fire('jumped_to', null);
        this.header.ready();
        //no stored state nothing to jump to
        return Promise.resolve(this);
    };
    CLUEWrapper.prototype.jumpToStoredOrLastState = function () {
        var _this = this;
        //jump to stored state
        var targetStory = this.clueManager.storedSlide;
        if (targetStory !== null) {
            return this.jumpToStory(targetStory);
        }
        var targetState = this.clueManager.storedState;
        if (targetState !== null) {
            return this.jumpToState(targetState);
        }
        return this.graph.then(function (graph) {
            var maxId = Math.max.apply(Math, graph.states.map(function (s) { return s.id; }));
            return _this.jumpToState(maxId);
        });
    };
    CLUEWrapper.prototype.reset = function () {
        var _this = this;
        this.graph.then(function (graph) {
            graph.jumpTo(graph.states[0]).then(function () {
                graph.clear();
                _this.$mainRef = graph.findOrAddObject(_this.$main, 'Application', 'visual');
                cmode.setMode(cmode.modes.Exploration);
            });
        });
    };
    return CLUEWrapper;
}(EventHandler));
export { CLUEWrapper };
/**
 * factory method creating a CLUEWrapper instance
 * @param body
 * @param options
 * @returns {CLUEWrapper}
 */
export function create(body, options) {
    if (options === void 0) { options = {}; }
    return new CLUEWrapper(body, options);
}
//# sourceMappingURL=template.js.map